/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_ACTION_EXECUTOR_H
#define FAULTD_ACTION_EXECUTOR_H

#include "event_processor.h"
#include "action_executed_event.h"

struct faultd_action {
	char *action_id;
	char *impl_name;

	struct nqueue_head execution_queue;
	struct sd_event_source *execution_queue_sd_source;

	/* execute first action from a queue */
	int (*execute)(struct faultd_action *act,
				   struct action_executed_event *exe_info);

	/* has to be initialized */
	struct list_head node;
};

int action_executor_action_register(struct faultd_action *act);
void action_executor_action_unregister(struct faultd_action *act);

#define FAULTD_ACTION_REGISTER(ACT, NAME)								\
static int action_ ##NAME## _init(struct faultd_module *module,			\
								  struct faultd_config *config,			\
										 sd_event *event_loop)			\
{																		\
	return action_executor_action_register(&ACT);						\
}																		\
																		\
static void action_ ##NAME## _cleanup(struct faultd_module *module)		\
{																		\
	action_executor_action_unregister(&ACT);							\
}																		\
																		\
static struct faultd_module action_ ##NAME## _module = {				\
	.name = #NAME,														\
	.type = FAULTD_MODULE_TYPE_ACTION,									\
	.init = action_ ##NAME## _init,										\
	.cleanup = action_ ##NAME## _cleanup,								\
	.node = LIST_HEAD_INIT(action_ ##NAME## _module.node),				\
};																		\
																		\
FAULTD_MODULE_REGISTER(&action_ ##NAME## _module)

#define FAULTD_ACTION_REGISTER_SIMPLE(ACT) \
	FAULTD_ACTION_REGISTER(ACT, ACT)

#endif /* FAULTD_ACTION_EXECUTOR_H */
