/*
 * This file is a part of fauld.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _DATABASE_H_
#define _DATABASE_H_

#include "common.h"
#include "module.h"
#include "event.h"

#define FAULTD_LAST_BOOT_OID "org.tizen.faultd.last_boot_oid"

struct faultd_database_adapter {
	struct faultd_module module;
	char *name;
	const char *id_key;

	/**
	 * @param[out] oid Will be filled with newly created oid
	 */
	int (*store)(struct faultd_database_adapter *adapter,
				 struct faultd_object *obj, faultd_oid_t *oid);

	int (*get_by_oid)(struct faultd_database_adapter *adapter,
					  faultd_oid_t *oid, struct faultd_object *result);

	int (*load)(struct faultd_database_adapter *adapter,
				struct faultd_object *query, struct faultd_object *hints,
				struct faultd_object *result, uint32_t *nr);

	int (*get_well_known_oid)(const char *name, faultd_oid_t *oid);

	bool (*is_oid_valid)(faultd_oid_t *oid);

	int (*get_event)(faultd_oid_t *oid, struct faultd_event **ev);
};

#define to_database_adapter(MOD)				\
	container_of(MOD, struct faultd_database_adapter, module)

struct faultd_database_adapter *faultd_get_database_adapter(void);
int faultd_set_database_adapter(struct faultd_database_adapter *adapter);
int database_store(struct faultd_object *obj, faultd_oid_t *oid);
int database_get_by_oid(faultd_oid_t *oid, struct faultd_object *result);
int database_load(struct faultd_object *query, struct faultd_object *hints,
				  struct faultd_object *result, uint32_t *nr);
int database_get_well_known_oid(const char *name, faultd_oid_t *oid);
bool database_is_oid_valid(faultd_oid_t *oid);
const char *database_get_id_key(void);
int database_get_event(faultd_oid_t *oid, struct faultd_event **ev);
int database_dump(struct faultd_object **out);

#endif	/* _DATABASE_H_ */
