/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_MODULE_H
#define FAULTD_MODULE_H

#include <time.h>
#include <systemd/sd-event.h>

#include "common.h"
#include "faultd-config.h"
#include "list.h"

#ifdef TEST_BUILD

#define TEST_ENTRY_POINT_NAME(MODULE, FUNC) \
	CONCAT(test_, CONCAT(MODULE, _ ## FUNC))

#define TEST_ENTRY_POINT(func) \
	typeof(func) (*CONCAT(FAULTD_MODNAME_T, _ ## func)) = func

#else
#define TEST_ENTRY_POINT(func)
#endif

#define FAULTD_MODNAME STR(FAULTD_MODNAME_T)

/* Order in this enum is also order of initialization */
enum faultd_module_type {
	FAULTD_MODULE_TYPE_MIN = 0,
	FAULTD_MODULE_TYPE_UNCATEGORIZED = FAULTD_MODULE_TYPE_MIN,
	FAULTD_MODULE_TYPE_CORE,
	FAULTD_MODULE_TYPE_DBADAPTER,
	FAULTD_MODULE_TYPE_EVENT,
	FAULTD_MODULE_TYPE_ACTION_EXECUTOR,
	FAULTD_MODULE_TYPE_ACTION,
	FAULTD_MODULE_TYPE_DECISION_MAKER,
	FAULTD_MODULE_TYPE_LISTENER,
	FAULTD_MODULE_TYPE_MAX,
};

struct faultd_module {
	char *name;
	enum faultd_module_type type;
	bool disabled;

	int (*init)(struct faultd_module *, struct faultd_config *config, sd_event *event_loop);
	void (*cleanup)(struct faultd_module *);

	struct list_head node;
};

int faultd_module_register(struct faultd_module *module);
void faultd_module_unregister(struct faultd_module *module);

int faultd_modules_init(sd_event *event_loop, struct faultd_config *conf);
void faultd_modules_cleanup(void);

struct faultd_module *faultd_get_module_by_name(const char *name);
void faultd_disable_module_category(int category);

int faultd_modules_load(void);
void faultd_modules_unload(void);

#ifndef __CONSTRUCTOR__
#define __CONSTRUCTOR__ __attribute__((constructor))
#endif

#ifndef __DESTRUCTOR__
#define __DESTRUCTOR__ __attribute__((destructor))
#endif

#define FAULTD_MODULE_REGISTER(module)				\
	static void __CONSTRUCTOR__ module_init(void)	\
	{												\
		faultd_module_register(module);				\
	}												\
	static void __DESTRUCTOR__ module_cleanup(void)	\
	{												\
		faultd_module_unregister(module);			\
	}

#endif /* FAULTD_MODULE_H */
