/*
 * This file is a part of fauld.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "database.h"

#define NOP_DEFAULT_OID "NOP"
#define NOP_ID_KEY_NAME "id"

struct nop_adapter {
	struct faultd_database_adapter database_adapter;
};

#define to_nop_adapter(MOD)				\
	container_of(MOD, struct nop_adapter, database_adapter)


static int init_nop_adapter(struct faultd_module *module,
							struct faultd_config *config,
							sd_event *event_loop)
{
	faultd_set_database_adapter(to_database_adapter(module));
	return 0;
}

static void cleanup_nop_adapter(struct faultd_module *module)
{
}

static int nop_get_by_oid(struct faultd_database_adapter *adapter,
						  faultd_oid_t *oid, struct faultd_object *result)
{
	return -ENOENT;
}

static int nop_load(struct faultd_database_adapter *adapter,
					struct faultd_object *query,
					struct faultd_object *hints,
					struct faultd_object *result, uint32_t *nr)
{
	*nr = 0;
	return 0;
}

static int nop_store(struct faultd_database_adapter *adapter,
					 struct faultd_object *obj, faultd_oid_t *oid)
{
	int ret;

	ret = faultd_object_get_oid(obj, NOP_ID_KEY_NAME, oid);
	if (ret == -ENOENT) {
		ret = strncpy(oid->nop, NOP_DEFAULT_OID, OID_NOP_SIZE);
	}

	return ret;
}


static int nop_get_well_known_oid(const char *name, faultd_oid_t *oid)
{
	return strncpy(oid->nop, NOP_DEFAULT_OID, OID_NOP_SIZE);
}

static bool nop_is_oid_valid(faultd_oid_t *oid)
{
	return strncmp(oid->nop, NOP_DEFAULT_OID, OID_NOP_SIZE) == 0;
}

static int nop_get_event(faultd_oid_t *oid, struct faultd_event **ev)
{
	return -ENOENT;
}

static struct nop_adapter nop_adapter = {
	.database_adapter = {
		.module = {
			.name = "nop_dbadapter",
			.type = FAULTD_MODULE_TYPE_DBADAPTER,
			.init = init_nop_adapter,
			.cleanup = cleanup_nop_adapter,
			.node = LIST_HEAD_INIT(nop_adapter.database_adapter.module.node),
		},
		.id_key = NOP_ID_KEY_NAME,
		.store = nop_store,
		.get_by_oid = nop_get_by_oid,
		.load = nop_load,
		.get_well_known_oid = nop_get_well_known_oid,
		.is_oid_valid = nop_is_oid_valid,
		.get_event = nop_get_event,
	}
};

FAULTD_MODULE_REGISTER(&nop_adapter.database_adapter.module);
