#include <errno.h>
#include <systemd/sd-id128.h>

#include "faultd_started_event.h"

static int fs_event_allocate(struct faultd_event_type *type,
							 void *data, struct faultd_event **ev)
{
	struct faultd_started_event *fs_ev;
	int ret;

	fs_ev = malloc(sizeof(*fs_ev));
	if (!fs_ev)
		return -ENOMEM;

	ret = faultd_event_init_internal(type, &fs_ev->event);
	if (ret)
		goto free_fs_ev;

	*ev = &fs_ev->event;
	return 0;

free_fs_ev:
	free(fs_ev);
	return ret;
}

static int deserialize_fs_event(struct faultd_event_type *type,
								struct faultd_object *data, struct faultd_event **ev)
{
	int ret = -EINVAL;

	ret = fs_event_allocate(type, NULL, ev);
	if (ret < 0)
		goto finish;

	ret = faultd_event_deserialize_internal(data, type, *ev);
	if (ret < 0) {
		struct faultd_started_event *fs_ev =
			to_faultd_started_event(*ev);
		free(fs_ev);
		goto finish;
	}

	ret = 0;
finish:
	return ret;
}
TEST_ENTRY_POINT(deserialize_fs_event);

static void fs_event_release(struct faultd_event *ev)
{
	struct faultd_started_event *fs_ev =
		to_faultd_started_event(ev);

	faultd_event_cleanup_internal(&fs_ev->event);
	free(fs_ev);
}

static void fs_event_serialize(struct faultd_event *ev, struct faultd_object *out)
{
	faultd_event_serialize_internal(ev, out);
}
TEST_ENTRY_POINT(fs_event_serialize);

static struct faultd_event_type faultd_started_event_type = {
	.name = FAULTD_STARTED_EVENT_ID,
	.default_ops = {
		.release = fs_event_release,
		.serialize = fs_event_serialize,
	},
	.allocate_event = fs_event_allocate,
	.deserialize_event = deserialize_fs_event,
	.node = LIST_HEAD_INIT(faultd_started_event_type.node),
};

FAULTD_EVENT_TYPE_REGISTER(faultd_started_event_type, faultd_started_et)
