/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_RESOURCE_VIOLATION_EVENT_H
#define FAULTD_RESOURCE_VIOLATION_EVENT_H

#include <time.h>

#include "event.h"
#include "common.h"
#include "service.h"

#define RESOURCE_VIOLATION_EVENT_ID "resource_violation"
#define RV_EV_DTIME "dt"
#define RV_EV_PID "pid"
#define RV_EV_RES_TYPE "rt"

enum resource_type {
	FAULTD_RESOURCE_FD,
	FAULTD_RESOURCE_MEM
};

struct resource_violation_event {
	struct faultd_event event;
	struct systemd_service service;
	time_t detection_time;
	enum resource_type resource_type;
	/* TODO: what more do we need? */
	pid_t pid;
};

struct rv_event_data {
	struct systemd_service service;
	time_t detection_time;
	enum resource_type resource_type;
	pid_t pid;
};

#define to_resource_violation_event(EVENT)						\
	container_of(EVENT, struct resource_violation_event, event)

#endif /* FAULTD_RESOURCE_VIOLATION_EVENT_H */
