#include "faultd_started_event.h"
#include "helpers.h"

#define TESTED_MOD faultd_started_event

DECLARE_TEST_ENTRY_POINT(TESTED_MOD, fs_event_serialize, void);
DECLARE_TEST_ENTRY_POINT(TESTED_MOD, deserialize_fs_event, int);

static struct faultd_event *alloc_event()
{
	struct faultd_started_event *fs_ev;

	fs_ev = calloc(1, sizeof(*fs_ev));
	assert_non_null(fs_ev);

	fill_event(&fs_ev->event);

	return &fs_ev->event;
}

static void generate_test_attrs(struct serialize_test_data *td)
{
	struct faultd_started_event *fs_ev = to_faultd_started_event(td->ev);

	struct serialized_element serialized_attrs[] = {
		S_ELEMENT_EVENT(fs_ev->event),
	};

	td->test_data = serialized_elems_dup(serialized_attrs,
										 ARRAY_SIZE(serialized_attrs));
	td->size = ARRAY_SIZE(serialized_attrs);
}

static int setup(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, fs_event_serialize),
									 NULL);
}

static int setup_deserialization(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, fs_event_serialize),
									 (deserialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, deserialize_fs_event));
}

static int teardown(void **state)
{
	struct serialize_test_data *td = *state;
	struct faultd_started_event *fs_ev = to_faultd_started_event(td->ev);

	free_event(&fs_ev->event);
	free(fs_ev);
	free_test_data(td);

	return 0;
}

FAULTD_TEST_GROUP(
	FAULTD_TEST_CASE_NST("fs_ev_serialize_test", test_event_serialization,
						 setup, teardown),
	FAULTD_TEST_CASE_NST("fs_ev_deserialize_test", test_event_deserialization,
						 setup_deserialization, teardown)
)
